/* <T.T.S.L.I Tree-tuple Synchronized languages Implementation >
*Copyright (C) <2002>  <LECLAND Benoit>

*This program is free software; you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation; either version 2 of the License, or
*(at your option) any later version.

*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.

*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package data;

import java.util.*;
import java.awt.*;

/** 
* Rpresente un arbre n-aire. 
*/ 
public class arbre{ 
    /** 
    * Libell de la racine. 
    */ 
    private String valeur; 
    /** 
    * Ensemble des sous-arbres. 
    */ 
    private sous_arbres sous_arbres; 
    /** 
    * Indique si cet arbre  t tudi. 
    */ 
    private boolean marque; 
    /** 
    * Cre un nouvel arbre constitu d'une racine sans sous-arbre. 
    * @param valeur Libell de la racine. 
    */ 
    public arbre(String valeur){ 
	    this.valeur = valeur; 
	    sous_arbres = new sous_arbres(); 
	    marque = false; 
    } 
    /** 
    * Cre un nouvel arbre vide (utilis par les classes descendantes). 
    */ 
    public arbre(){ 
	marque = false; 
    } 
    /** 
    * Ajout d'un sous-arbre. 
    * @param sous_arbre Le sous-arbre ajout. 
    */ 
    public void addSousArbre(arbre sous_arbre){ 
	    this.sous_arbres.addArbre(sous_arbre); 
    } 
    /** 
    * Ajout d'un sous-arbre. 
    * @param valeur Libell de la racine du sous-arbre ajout. 
    */ 
    public void addSousArbre(String valeur){ 
	    addSousArbre(new arbre(valeur)); 
    } 
    /** 
    * Retourne le libell de la racine. 
    */ 
    public String getValeur(){ 
	    return valeur; 
    } 
    /** 
    * Modifie la valeur de la racine. 
    */ 
    public void setValeur(String valeur){ 
	    this.valeur = valeur; 
    } 
    /** 
    * Nouvel ensemble de sous-arbres. 
    */ 
    private void setSousArbres(sous_arbres sa){ 
    	sous_arbres=sa; 
    } 
    /** 
    * Retourne l'ensemble des sous-arbres. 
    */ 
    public sous_arbres getSousArbres(){

    	return sous_arbres; 
    } 
    /** 
    * Retourne le i-me sous arbre de l'ensemble des sous-arbres. 
    */ 
    public arbre getSousArbre(int i){ 
    	return getSousArbres().getArbre(i); 
    } 
    /** 
    * Indique si l'arbre  t tudi. 
    */ 
    public boolean getMarque(){ 
	return marque; 
    } 
    /** 
    * Change la valeur de marque. 
    * @param marque Nouvelle valeur de marque.  
    */ 
    public void setMarque(boolean marque){ 
	this.marque = marque; 
    }
    /**
    * Indique si l'arbre contient un libell donn.
    * @param lib Libell  recherch.
    */
    public boolean contains(String lib){
	boolean retour = lib.equals(getValeur());
	if(!retour){
	    retour = getSousArbres().contains(lib);
	}
	return retour;
    }
    /** 
    * Indique si 2 arbres sont gaux (sans permutation). 
    * @param a l'arbre avec lequel l'objet courant est compar. 
    */ 
    public boolean equals(arbre a){ 
	boolean retour = true; 
	if(getValeur().equals(a.getValeur())){ 
	    retour = getSousArbres().equals(a.getSousArbres()); 
	}//if 
	else{ 
	    retour = false;} 
	return retour; 
    } 
    /** 
    * Indique si 2 arbres sont gaux (avec permutation). 
    * @param a l'arbre avec lequel l'objet courant est compar. 
    */ 
    public boolean equals_permutation(arbre a){ 
	boolean retour = true; 
	if(getValeur().equals(a.getValeur())){ 
	    retour = getSousArbres().equals_permutation(a.getSousArbres()); 
	}//if 
	else{ 
	    retour = false;} 
	return retour; 
    } 
    /** 
    * Retourne l'arbre sous forme d'une chaine de caractre. 
    */ 
    public String print(){ 
	    String retour =valeur; 
	    if(this.sous_arbres.size()>0){ 
	        retour +="("; 
	        for(int i=0;i<this.sous_arbres.size();i++){ 
    	        retour += this.sous_arbres.getArbre(i).print(); 
    	        if(i+1<this.sous_arbres.size()) 
    	            retour+=","; 
    	    } 
    	    retour+=")";    
	    }	     
    	return retour; 
    } 
    /** 
    * Retourne une copie de l'arbre. 
    */ 
    public arbre copy(){ 
	//return (new expression(print()).getArbre()); 
	arbre retour = new arbre(getValeur());         
	retour.setMarque(getMarque()); 
	if(getSousArbres().size()>0){ 
	    retour.setSousArbres(getSousArbres().copy());} 
	return retour; 
    }     
    /**
    * Renommage lors de la dterminisation.
    */
    public void rename(String[] old_lib, String[] new_lib){
	for(int i=0;i<old_lib.length;i++){
	    if(!new_lib[i].equals("") && getValeur().equals(old_lib[i])){
		valeur = new String(new_lib[i]);
		break;
	    }
	}
	if(getSousArbres().size()>0){
	    getSousArbres().rename(old_lib, new_lib);
	}
    }
    /*------------------Gestion de l'affichage------------------*/ 
    /** 
    * Gestion de l'affichage: Retourne la largeur de l'arbre en pixel. 
    * @param fontMetrics Pour connaitre la dimension des libells. 
    */ 
    public int largeur(FontMetrics fontMetrics){ 
	    int retour = 0; 
	    int largeur_sous_arbres = sous_arbres.largeur(fontMetrics); 
	    if(largeurValeur(fontMetrics)>largeur_sous_arbres) 
	        retour = largeurValeur(fontMetrics); 
	    else 
	        retour = largeur_sous_arbres; 
	    return retour; 
    }    
    /** 
    * Gestion de l'affichage: Retourne la largeur en pixel du libell. 
    * @param fontMetrics Pour connaitre la dimension des libells. 
    */ 
    public int largeurValeur(FontMetrics fontMetrics){ 
	    int largeur_libelle=fontMetrics.stringWidth(valeur); 
	    return largeur_libelle; 
    } 
    /** 
    * Gestion de l'affichage: Retourne la hauteur en pixel du libell. 
    * @param fontMetrics Pour connaitre la dimension des libells. 
    */ 
    public int hauteurValeur(FontMetrics fontMetrics){ 
    	int hauteur_libelle=fontMetrics.getHeight(); 
	    return hauteur_libelle; 
    } 
    /** 
    * Gestion de l'affichage: Retourne la hauteur de l'arbre en pixel. 
    * @param fontMetrics Pour connaitre la dimension des libells. 
    */ 
    public int hauteur(FontMetrics fontMetrics){ 
	int hauteur=hauteurValeur(fontMetrics); 
	int hauteur_sous_arbres=0; 
	for(int i=0;i<sous_arbres.size();i++){ 
	    int hauteur_sous_arbres_i = getSousArbre(i).hauteur(fontMetrics); 
	    if(hauteur_sous_arbres<hauteur_sous_arbres_i) 
		hauteur_sous_arbres=hauteur_sous_arbres_i; 
	} 
	int retour=hauteur; 
	if(hauteur_sous_arbres>0) 
	    retour+=30+hauteur_sous_arbres; 
	return retour; 
    }    
    /** 
    * Gestion de l'affichage: Dessine l'arbre dans une zone graphique. 
    * @param g Zone graphique dans laquelle l'arbre est affich. 
    * @param fonMetrics Pour connaitre la dimension des libells. 
    * @param x Abscisse du coin suprieur gauche. 
    * @param y Ordonne du coin suprieur gauche. 
    */ 
    public void draw(Graphics g, FontMetrics fontMetrics, int x, int y){ 
    	int x_racine = (largeur(fontMetrics)-largeurValeur(fontMetrics))/2 + x; 
	    int xl = x + largeur(fontMetrics)/2; 
	    int yl = y +3; 
	    if(getMarque()) 
	        g.setColor(Color.red);	         
	    g.drawString(getValeur(),x_racine,y); 
	    g.setColor(Color.black); 
	    if(getSousArbres().size()>0){ 
    	    int x0 = x; 
    	    //Si l'unique sous-arbre est plus troit que l'tiquette de l'arbre: 
    	    if(largeurValeur(fontMetrics)>getSousArbres().largeur(fontMetrics)){    	         
    	        x0 = x + (largeurValeur(fontMetrics)/2 - getSousArbres().largeur(fontMetrics)/2);} 
	        int y0 = y+40; 
	        int x0l = x0 + getSousArbre(0).largeur(fontMetrics)/2; 
	        int y0l = y0 - getSousArbre(0).hauteurValeur(fontMetrics); 
	        g.drawLine(xl,yl,x0l,y0l); 
	        getSousArbre(0).draw(g,fontMetrics,x0,y0); 
	        int xavant = x0; 
	        for(int i=1;i<getSousArbres().size();i++){ 
		        int xi=xavant+getSousArbre(i-1).largeur(fontMetrics)+10; 
		        xavant = xi; 
		        int yi=y+40; 
		        int xil= xi + getSousArbre(i).largeur(fontMetrics)/2; 
		        int yil= yi - getSousArbre(i).hauteurValeur(fontMetrics); 
		        g.drawLine(xl,yl,xil,yil); 
		        getSousArbre(i).draw(g,fontMetrics,xi,yi); 
	        } 
	    } 
    }    
}
