/* <T.T.S.L.I Tree-tuple Synchronized languages Implementation >
*Copyright (C) <2002>  <LECLAND Benoit>

*This program is free software; you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation; either version 2 of the License, or
*(at your option) any later version.

*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.

*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package data;

import java.util.*;

/**   
* Reprsente un ensemble d'tat.   
*/   
public class ensembleEtats extends Vector {   
    /**   
    * Cre un nouvel ensemble d'tat vide.   
    */   
    public ensembleEtats(){   
	super();   
    }   
    /**   
    * Ajoute un tat  l'ensemble.   
    * @param e Objet etat ajout dans l'ensemble.   
    */   
    public void addEtat(etat e){
	if(!contains(e))
	    addElement(e);   
    }   
    /**   
    * Ajoute un tat non final  l'ensemble.   
    * @param e Libell de l'tat ajout dans l'ensemble.   
    */   
    public void addEtat(String e){   
	addEtat(new etat(e));   
    }   
    /**   
    * Ajoute un tat  l'ensemble.   
    * @param e Libell de l'tat.   
    * @param f Indique si l'tat est final.   
    */   
    public void addEtat(String e, boolean f){   
	addEtat(new etat(e,f));   
    }   
    /**   
    * Retourne le i-me tat.   
    * @param i Index de l'tat.   
    */   
    public etat getEtat(int i){   

	return (etat)elementAt(i);   
    }               
    /**   
    * Retourne un tat.   
    * @param Libell de l'tat voulu.   
    */   
    public etat getEtat(String e){   
	return getEtat(getEtatIndex(e));   
    }   
    /**   
    * Retourne l'index d'un tat.   
    * @param e Libell de l'tat.   
    */   
    public int getEtatIndex(String e){   
	int retour=-1;   
	for(int i=0;i<size();i++){   
	    if(getLibelle(i).equals(e)){   
		retour = i;		   
		break;   
	    }   
	}   
	return retour;   
    }   
    /**   
    * Retourne le libell du i-me tat.   
    * @param i Index de l'tat voulu.   
    */   
    public String getLibelle(int i){   
	return getEtat(i).getLibelle();   
    }   
    /**   
    * Change la condition "final" du i-me tat.   
    * @param i Index de l'tat.   
    * @param eFinal Nouvelle valeur de final.   
    */   
    public void setFinal(int i,boolean eFinal){   
	getEtat(i).setFinal(eFinal);   
    }   
    /**   
    * Indique si le i-me tat est final.       
    * @param i Index de l'tat.   
    */   
    public boolean isFinal(int i){   

	return getEtat(i).isFinal();  
	    }  
    /**  
     * Indique si un tat est final.  
     * @param etat Libell de l'tat.  
     */  
    public boolean isFinal(String etat){  
	return isFinal(getEtatIndex(etat));  
    }
    /**
    * Ensemble des tats finaux.
    */
    private ensembleEtats getFinals(){
	ensembleEtats finals = new ensembleEtats();
	for(int i=0;i<size();i++){
	    if(isFinal(i))
		finals.addEtat(getEtat(i));
	}
	return finals;
    }    
    /**
    * Retourne l'ensemble sous forme de chaine.
    */
     public String print(){
	String retour = new String("ETATS = { ");
	for(int i=0;i<size();i++){
	    retour = retour + getEtat(i).getLibelle();
	    if(i!=size()-1)
		retour = retour+" , ";
	}
	retour = retour + " }\n\nETATS FINAUX = { ";
	ensembleEtats Qf = getFinals();
	for(int j=0;j<Qf.size();j++){
	     retour = retour + Qf.getEtat(j).getLibelle();
	    if(j!=Qf.size()-1)
		retour = retour+" , ";
	}
	retour = retour + " }\n";
	return retour;
    }
    /**
    * Retourne une copie de l'ensemble.
    */
    public ensembleEtats copy(){
	ensembleEtats retour = new ensembleEtats();
	for(int i=0;i<size();i++)
	    retour.addEtat(getEtat(i).copy());
	return retour;
    }
    /**
    * Tri les tats dans l'ordre alphabetique du liblle.
    */
    public void sort(){
	ensembleEtats trie = new ensembleEtats();	
	while(size()>0){
	    int max = 0;
	    for(int i=1;i<size();i++){
		if(getEtat(i).getLibelle().compareTo(getEtat(max).getLibelle())>0)
		    max = i;
	    }
	    trie.insertElementAt(getEtat(max),0);
	    removeElementAt(max);
	}
	for(int i=0;i<trie.size();i++)
	    addEtat(trie.getEtat(i));
    }
    /**
    * Retourne une chaine : {Q0,...,Qn} sans indication des tats finaux.
    */
    public String getEnsemble(){
	String retour = new String("{");
	for(int i=0;i<size();i++){
	    retour = retour + getEtat(i).getLibelle();	    
	    if(i!=size()-1)
		retour = retour+",";
	}
	retour = retour + "}";
	return retour;
    }
    /**
    * Indique si un tat est dj prsent.
    */
    public boolean contains(etat e){
	boolean retour = false;
	for(int i=0;i<size();i++){
	  
	    if(getEtat(i).equals(e)){
		
		retour = true;
		break;
	    }
	}	
	return retour;
    }
    /**
    * Indique si un tat est dj prsent mais ne verifie pas qu'il est final 
    */ 
    public boolean contains2(String s){
	boolean retour = false;
	etat e1 = new etat(s,false);
	etat e2 = new etat(s,true);
	if ( contains(e1) || contains(e2) ) 
	    retour = true;
	   
	
	return retour;
    }
 
}   
 
