/* <T.T.S.L.I Tree-tuple Synchronized languages Implementation >
*Copyright (C) <2002>  <LECLAND Benoit>

*This program is free software; you can redistribute it and/or modify
*it under the terms of the GNU General Public License as published by
*the Free Software Foundation; either version 2 of the License, or
*(at your option) any later version.

*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.

*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package data;

import java.util.*;

/** 
* Reprsente un ensemble de fonctions de transition. 
*/ 
public class ensembleTransitions extends Vector{ 
    /**
    * Indique si la prsence de 2 lments identiques est possible.
    */
    private boolean list;
    /** 
    * Cre un nouvel ensemble de transitions vide. 
    */ 
    public ensembleTransitions(){ 
	super(); 
	list = false;
    }
    /** 
    * Cre un nouvel ensemble de transitions vide. 
    * @param list Indique si la prsence de 2 lments identiques est possible.
    */ 
    public ensembleTransitions(boolean list){ 
	super(); 
	this.list = list;
    }
    /** 
    * Ajoute une rgle de transition dans l'ensemble. 
    * @param t Rgle ajoute. 
    */ 
    public void addTransition(transition t){
	if(!list){
	    if(!contains(t))
		addElement(t); }
	else
	    addElement(t);
    } 
    /** 
    * Ajoute une rgle de transition dans l'ensemble. 
    * @param gauche Partie gauche de la rgle (une expression). 
    * @param droite Partie droite de la rgle (un tat). 
    */ 
    public void addTransition(expression gauche, etat droite){ 
	addTransition(new transition(gauche, droite)); 
    } 
    /** 
    * Ajoute une rgle de transition dans l'ensemble. 
    * @param gauche Partie gauche de la rgle (une expression sous forme de chaine). 
    * @param droite Partie droite de la rgle (un tat). 
    */ 
    public void addTransition(String gauche, etat droite){ 
	addTransition(new expression(gauche), droite); 
    } 
    /** 
    * Retourne la i-me transition de l'ensemble 
    * @param i Index de la transition. 
    */ 
    public transition getTransition(int i){ 
	return (transition)elementAt(i); 
    } 
    /** 
    * Retourne la partie gauche de la i-me rgle.     
    * @param i l'index de la rgle. 
    */ 
    public expression getCondition(int i){ 
	return getTransition(i).getCondition(); 
    } 
    /** 
    * Retourne la partir droite de la i-me rgle. 
    * @param i l'index de la rgle. 
    */ 
    public etat getEtatObtenu(int i){ 
	return getTransition(i).getEtatObtenu(); 
    } 
    /** 
    * Indique si la i-me rgle de transition est satisfaite par un arbre. 
    * @param i l'index de la rgle. 
    * @param a L'arbre  tudier. 
    */ 
    public boolean obtenue(int i, arbre a){ 
	return getTransition(i).getCondition().getArbre().equals(a); 
    } 
    /** 
    * Indique si l'ensemble contient des rgles possdant la meme partie gauche. 
    */ 
    public boolean left_equals(){ 
	boolean retour = false; 
	for(int i=0;i<size();i++){ 
	    for(int j=0;j<size();j++){ 
		if( (i!=j) && getTransition(i).left_equals(getTransition(j))){
		    retour = true; 
		    break; 
		} 
	    } 
	} 
	return retour; 
    }
    /**
    * Indique si une transition est dj prsente dans l'ensemble.
    */
    public boolean contains(transition t){
	boolean retour = false;
	for(int i=0;i<size();i++){
	    if(getTransition(i).equals(t)){
		retour = true;
		break;
	    }
	}
	return retour;
    }

    public int contient(transition t){
	int retour = -1;
	for(int i=0;i<size();i++){
	    if(getTransition(i).print().equals(t.print())){
		retour = i;
		break;
	    }
	}
	return retour;
    }

    /** 
    * Retourne l'ensemble de rgles sous forme de chaine. 
    */ 
    public String print(){ 
	String retour = new String(); 
	if(size()>0) 
	    retour = "REGLES DE TRANSITIONS = {\n "+getTransition(0).print();         
	for(int i=1;i<size();i++) 
	    retour +=" ,\n "+getTransition(i).print(); 
	return retour+" }\n"; 
    } 
    /**
    /**
    * Retourne une copie de l'ensemble.
    */
    public ensembleTransitions copy(){
	ensembleTransitions retour = new ensembleTransitions();
	for(int i=0;i<size();i++){
	    retour.addTransition(getTransition(i).copy());
	}
	return retour;
    }
} 
